@@ -77,9 +77,22 @@ struct _GdkDisplayX11
 
   gboolean use_xshm;
   gboolean have_shm_pixmaps;
-  GdkTristate gravity_works;
   GdkTristate have_render;
-  
+  gboolean have_xfixes;
+  gint xfixes_event_base;
+
+  gboolean have_xcomposite;
+  gboolean have_xdamage;
+  gint xdamage_event_base;
+
+  gboolean have_randr13;
+  gint xrandr_event_base;
+
+  /* If the SECURITY extension is in place, whether this client holds 
+   * a trusted authorization and so is allowed to make various requests 
+   * (grabs, properties etc.) Otherwise always TRUE. */
+  gboolean trusted_client;
+
   /* Information about current pointer and keyboard grabs held by this
    * client. If gdk_pointer_xgrab_window or gdk_keyboard_xgrab_window
    * window is NULL, then the other associated fields are ignored
@@ -87,10 +100,13 @@ struct _GdkDisplayX11
   GdkWindowObject *pointer_xgrab_window;
   gulong pointer_xgrab_serial;
   gboolean pointer_xgrab_owner_events;
+  gboolean pointer_xgrab_implicit;
+  guint32 pointer_xgrab_time;
 
   GdkWindowObject *keyboard_xgrab_window;
   gulong keyboard_xgrab_serial;
   gboolean keyboard_xgrab_owner_events;
+  guint32 keyboard_xgrab_time;
 
   /* drag and drop information */
   GdkDragContext *current_dest_drag;
@@ -109,6 +125,7 @@ struct _GdkDisplayX11
 
   /* Session Management leader window see ICCCM */
   Window leader_window;
+  GdkWindow *leader_gdk_window;
   gboolean leader_window_title_set;
   
   /* list of filters for client messages */
@@ -138,10 +155,20 @@ struct _GdkDisplayX11
   /* Startup notification */
   gchar *startup_notification_id;
 
+  /* Time of most recent user interaction. */
+  gulong user_time;
+
   /* Sets of atoms for DND */
   guint base_dnd_atoms_precached : 1;
   guint xdnd_atoms_precached : 1;
   guint motif_atoms_precached : 1;
+  guint use_sync : 1;
+
+  guint have_shapes : 1;
+  guint have_input_shapes : 1;
+
+  /* Alpha mask picture format */
+  XRenderPictFormat *mask_format;
 };
 
 struct _GdkDisplayX11Class
