@@ -24,18 +24,18 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <string.h>
 
-#include "gdkalias.h"
 #include "gdkx.h"
 #include "gdkproperty.h"
 #include "gdkselection.h"
 #include "gdkprivate.h"
 #include "gdkprivate-x11.h"
 #include "gdkdisplay-x11.h"
+#include "gdkalias.h"
 
 typedef struct _OwnerInfo OwnerInfo;
 
@@ -104,7 +104,7 @@ _gdk_selection_filter_clear_event (XSelectionClearEvent *event)
 /**
  * gdk_selection_owner_set_for_display:
  * @display: the #GdkDisplay.
- * @owner: a #GdkWindow or %NULL to indicate that the the owner for
+ * @owner: a #GdkWindow or %NULL to indicate that the owner for
  *         the given should be unset.
  * @selection: an atom identifying a selection.
  * @time_: timestamp to use when setting the selection. 
@@ -250,7 +250,7 @@ gdk_selection_convert (GdkWindow *requestor,
        If the retrieval failed, %NULL we be stored here, otherwise, it
        will be non-%NULL and the returned data should be freed with g_free()
        when you are finished using it. The length of the
-       allocated memory is one more than the the length
+       allocated memory is one more than the length
        of the returned data, and the final byte will always
        be zero, to ensure nul-termination of strings.
  * @prop_type: location to store the type of the property.
@@ -387,12 +387,12 @@ gdk_selection_property_get (GdkWindow  *requestor,
  * Since: 2.2
  **/
 void
-gdk_selection_send_notify_for_display (GdkDisplay *display,
-				       guint32     requestor,
-				       GdkAtom     selection,
-				       GdkAtom     target,
-				       GdkAtom     property, 
-				       guint32     time)
+gdk_selection_send_notify_for_display (GdkDisplay       *display,
+				       GdkNativeWindow  requestor,
+				       GdkAtom          selection,
+				       GdkAtom          target,
+				       GdkAtom          property, 
+				       guint32          time)
 {
   XSelectionEvent xevent;
   
@@ -404,7 +404,10 @@ gdk_selection_send_notify_for_display (GdkDisplay *display,
   xevent.requestor = requestor;
   xevent.selection = gdk_x11_atom_to_xatom_for_display (display, selection);
   xevent.target = gdk_x11_atom_to_xatom_for_display (display, target);
-  xevent.property = gdk_x11_atom_to_xatom_for_display (display, property);
+  if (property == GDK_NONE)
+    xevent.property = None;
+  else
+    xevent.property = gdk_x11_atom_to_xatom_for_display (display, property);
   xevent.time = time;
 
   _gdk_send_xevent (display, requestor, False, NoEventMask, (XEvent*) & xevent);
@@ -536,10 +539,11 @@ make_list (const gchar  *text,
     }
 
   if (list)
-    *list = g_new (gchar *, n_strings + 1);
-
-  (*list)[n_strings] = NULL;
-  
+    {
+      *list = g_new (gchar *, n_strings + 1);
+      (*list)[n_strings] = NULL;
+    }
+     
   i = n_strings;
   tmp_list = strings;
   while (tmp_list)
@@ -548,10 +552,10 @@ make_list (const gchar  *text,
 	(*list)[--i] = tmp_list->data;
       else
 	g_free (tmp_list->data);
-
+      
       tmp_list = tmp_list->next;
     }
-
+  
   g_slist_free (strings);
 
   return n_strings;
@@ -591,7 +595,7 @@ gdk_text_property_to_utf8_list_for_display (GdkDisplay    *display,
     {
       return make_list ((gchar *)text, length, TRUE, list);
     }
-  else if (encoding == gdk_atom_intern ("UTF8_STRING", FALSE))
+  else if (encoding == gdk_atom_intern_static_string ("UTF8_STRING"))
     {
       return make_list ((gchar *)text, length, FALSE, list);
     }
@@ -675,7 +679,7 @@ gdk_text_property_to_utf8_list_for_display (GdkDisplay    *display,
  * Convert a string from the encoding of the current 
  * locale into a form suitable for storing in a window property.
  * 
- * Returns: 0 upon sucess, non-zero upon failure. 
+ * Returns: 0 upon success, non-zero upon failure. 
  *
  * Since: 2.2
  **/
@@ -726,7 +730,8 @@ gdk_string_to_compound_text_for_display (GdkDisplay  *display,
  * from the input string and also canonicalizes \r, and \r\n to \n
  */
 static gchar * 
-sanitize_utf8 (const gchar *src)
+sanitize_utf8 (const gchar *src,
+	       gboolean return_latin1)
 {
   gint len = strlen (src);
   GString *result = g_string_sized_new (len);
@@ -745,13 +750,26 @@ sanitize_utf8 (const gchar *src)
       else
 	{
 	  gunichar ch = g_utf8_get_char (p);
-	  char buf[7];
-	  gint buflen;
 	  
 	  if (!((ch < 0x20 && ch != '\t' && ch != '\n') || (ch >= 0x7f && ch < 0xa0)))
 	    {
-	      buflen = g_unichar_to_utf8 (ch, buf);
-	      g_string_append_len (result, buf, buflen);
+	      if (return_latin1)
+		{
+		  if (ch <= 0xff)
+		    g_string_append_c (result, ch);
+		  else
+		    g_string_append_printf (result,
+					    ch < 0x10000 ? "\\u%04x" : "\\U%08x",
+					    ch);
+		}
+	      else
+		{
+		  char buf[7];
+		  gint buflen;
+		  
+		  buflen = g_unichar_to_utf8 (ch, buf);
+		  g_string_append_len (result, buf, buflen);
+		}
 	    }
 
 	  p = g_utf8_next_char (p);
@@ -778,21 +796,7 @@ sanitize_utf8 (const gchar *src)
 gchar *
 gdk_utf8_to_string_target (const gchar *str)
 {
-  GError *error = NULL;
-  
-  gchar *tmp_str = sanitize_utf8 (str);
-  gchar *result =  g_convert_with_fallback (tmp_str, -1,
-					    "ISO-8859-1", "UTF-8",
-					    NULL, NULL, NULL, &error);
-  if (!result)
-    {
-      g_warning ("Error converting from UTF-8 to STRING: %s",
-		 error->message);
-      g_error_free (error);
-    }
-  
-  g_free (tmp_str);
-  return result;
+  return sanitize_utf8 (str, TRUE);
 }
 
 /**
@@ -831,7 +835,7 @@ gdk_utf8_to_compound_text_for_display (GdkDisplay  *display,
 
   need_conversion = !g_get_charset (&charset);
 
-  tmp_str = sanitize_utf8 (str);
+  tmp_str = sanitize_utf8 (str, FALSE);
 
   if (need_conversion)
     {
@@ -880,3 +884,6 @@ void gdk_free_compound_text (guchar *ctext)
   if (ctext)
     XFree (ctext);
 }
+
+#define __GDK_SELECTION_X11_C__
+#include "gdkaliasdef.c"
